/******************************************************************************
 *
 * Copyright(c) 2007 - 2010 Realtek Corporation. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
 *
 * Modifications for inclusion into the Linux staging tree are
 * Copyright(c) 2010 Larry Finger. All rights reserved.
 *
 * Contact information:
 * WLAN FAE <wlanfae@realtek.com>
 * Larry Finger <Larry.Finger@lwfinger.net>
 *
 ******************************************************************************/
#ifndef __RTL8712_REGDEF_H__
#define __RTL8712_REGDEF_H__

#include "rtl8712_syscfg_regdef.h"
#include "rtl8712_cmdctrl_regdef.h"
#include "rtl8712_macsetting_regdef.h"
#include "rtl8712_timectrl_regdef.h"
#include "rtl8712_fifoctrl_regdef.h"
#include "rtl8712_ratectrl_regdef.h"
#include "rtl8712_edcasetting_regdef.h"
#include "rtl8712_wmac_regdef.h"
#include "rtl8712_powersave_regdef.h"
#include "rtl8712_gp_regdef.h"
#include "rtl8712_debugctrl_regdef.h"

#define HIMR	(RTL8712_INTERRUPT_ + 0x08)

#endif /* __RTL8712_REGDEF_H__*/

